#include <windows.h>
#include <stdio.h>

#include "win048.h"

/* Enumerados */
enum estado {libre, reservado, acomodado, bloqueado};

/* Declaraciones de tipos */
typedef struct stDatos {
   char Texto[2048];
} DATOS;

/*  Declaracin del procedimiento de ventana  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
void ActualizarMenu(HWND, enum estado);
void ActualizarVentana(HDC, DATOS *);


int WINAPI WinMain (HINSTANCE hThisInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpszArgument,
                    int nFunsterStil)
{
    HWND hwnd;               /* Manipulador de ventana */
    MSG mensaje;             /* Mensajes recibidos por la aplicacin */
    WNDCLASSEX wincl;        /* Estructura de datos para la clase de ventana */

    /* Estructura de la ventana */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = "NUESTRA_CLASE";
    wincl.lpfnWndProc = WindowProcedure;      /* Esta funcin es invocada por Windows */
    wincl.style = CS_DBLCLKS;                 /* Captura los doble-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Usar icono y puntero por defector */
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";
    wincl.cbClsExtra = 0;                      /* Sin informacin adicional para la */
    wincl.cbWndExtra = 0;                      /* clase o la ventana */
    /* Usar el color de fondo por defecto para la ventana */
    wincl.hbrBackground = GetSysColorBrush(COLOR_BACKGROUND);

    /* Registrar la clase de ventana, si falla, salir del programa */
    if(!RegisterClassEx(&wincl)) return 0;

    /* La clase est registrada, crear la ventana */
    hwnd = CreateWindowEx(
           0,                   /* Posibilidades de variacin */
           "NUESTRA_CLASE",     /* Nombre de la clase */
           "Ejemplo 048",       /* Texto del ttulo */
           WS_OVERLAPPEDWINDOW, /* Tipo por defecto */
           CW_USEDEFAULT,       /* Windows decide la posicin */
           CW_USEDEFAULT,       /* donde se coloca la ventana */
           544,                 /* Ancho */
           375,                 /* Alto en pixels */
           HWND_DESKTOP,        /* La ventana es hija del escritorio */
           NULL,                /* Sin men */
           hThisInstance,       /* Manipulador de instancia */
           NULL                 /* No hay datos de creacin de ventana */
    );

    /* Mostrar la ventana */
    ShowWindow(hwnd, SW_SHOWDEFAULT);

    /* Bucle de mensajes, se ejecuta hasta que haya error o GetMessage devuelva FALSE */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Traducir mensajes de teclas virtuales a mensajes de caracteres */
        TranslateMessage(&mensaje);
        /* Enviar mensaje al procedimiento de ventana */
        DispatchMessage(&mensaje);
    }

    /* Salir con valor de retorno */
    return mensaje.wParam;
}

/*  Esta funcin es invocada por la funcin DispatchMessage()  */
LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static DATOS *Datos;
    static enum estado estadoActual;
    HDC hdc;
    PAINTSTRUCT ps;
    int i;
    DWORD protant;

    switch (msg)                  /* manipulador del mensaje */
    {
        case WM_CREATE:
           Datos = NULL;
           estadoActual = libre;
           ActualizarMenu(hwnd, estadoActual);
           return 0;
        case WM_COMMAND:
           switch(LOWORD(wParam)) {
              case CM_RESERVAR:
                 Datos = VirtualAlloc(NULL, 25 * sizeof(DATOS),
                    MEM_RESERVE, PAGE_NOACCESS);
                 estadoActual = reservado;
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_ACOMODARESERVADA:
                 VirtualAlloc(Datos, 25*sizeof(DATOS),
                    MEM_COMMIT, PAGE_READWRITE);
                 for(i=0; i < 25; i++)
                    sprintf(Datos[i].Texto, "Cadena %d", i+1);
                 estadoActual = acomodado;
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_ACOMODAR:
                 Datos = VirtualAlloc(NULL, 25 * sizeof(DATOS),
                    MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
                 for(i=0; i < 25; i++)
                    sprintf(Datos[i].Texto, "Cadena %d", i+1);
                 estadoActual = acomodado;
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_BLOQUEAR:
                 if(VirtualLock(Datos, 25*sizeof(DATOS))) {
                    MessageBox(hwnd, "Bloqueo de memoria activo", "Ejemplo 048", MB_OK);
                    estadoActual = bloqueado;
                 }
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_DESBLOQUEAR:
                 if(VirtualUnlock(Datos, 25*sizeof(DATOS))) {
                    MessageBox(hwnd, "Bloqueo de memoria no activo", "Ejemplo 048", MB_OK);
                    estadoActual = acomodado;
                 }
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_LIBERAR:
                 VirtualFree(Datos, 0, MEM_RELEASE);
                 estadoActual = libre;
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_DESASIGNA:
                 VirtualFree(Datos, 25*sizeof(DATOS), MEM_DECOMMIT);
                 estadoActual = reservado;
                 ActualizarMenu(hwnd, estadoActual);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_SOLOLECTURA:
                 VirtualProtect(Datos, 25*sizeof(DATOS), PAGE_READONLY, &protant);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_LECTURAESCRITURA:
                 VirtualProtect(Datos, 25*sizeof(DATOS), PAGE_READWRITE, &protant);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_ESCRITURACOPIA:
                 VirtualProtect(Datos, 25*sizeof(DATOS), PAGE_WRITECOPY, &protant);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_EJECUCION:
                 VirtualProtect(Datos, 25*sizeof(DATOS), PAGE_EXECUTE, &protant);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_SINACCESO:
                 VirtualProtect(Datos, 25*sizeof(DATOS), PAGE_NOACCESS, &protant);
                 InvalidateRect(hwnd, NULL, TRUE);
                 break;
              case CM_SALIR:
                 DestroyWindow(hwnd);
                 break;
              case CM_VERMEMORIA:
                 hdc = GetDC(hwnd);
                 SetBkMode(hdc, TRANSPARENT);
                 for(i=0; i < 25; i++) {
                    TextOut(hdc, 300, 10+20*i, Datos[i].Texto, strlen(Datos[i].Texto));
                 }
                 break;
           }
           break;
        case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);
           ActualizarVentana(hdc, Datos);
           EndPaint(hwnd, &ps);
           break;
        case WM_DESTROY:
           PostQuitMessage(0);    /* enva un mensaje WM_QUIT a la cola de mensajes */
           break;
        default:                  /* para los mensajes de los que no nos ocupamos */
           return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void ActualizarMenu(HWND hwnd, enum estado actual)
{
   int i;

   switch(actual) {
      case libre:
         EnableMenuItem(GetMenu(hwnd), CM_RESERVAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODARESERVADA, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_BLOQUEAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESBLOQUEAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_LIBERAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESASIGNA, MF_GRAYED | MF_BYCOMMAND);
         for(i=CM_SOLOLECTURA; i <= CM_SINACCESO; i++)
            EnableMenuItem(GetMenu(hwnd), i, MF_GRAYED | MF_BYCOMMAND);
         break;
      case reservado:
         EnableMenuItem(GetMenu(hwnd), CM_RESERVAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODARESERVADA, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_BLOQUEAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESBLOQUEAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_LIBERAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESASIGNA, MF_GRAYED | MF_BYCOMMAND);
         for(i=CM_SOLOLECTURA; i <= CM_SINACCESO; i++)
            EnableMenuItem(GetMenu(hwnd), i, MF_GRAYED | MF_BYCOMMAND);
         break;
      case acomodado:
         EnableMenuItem(GetMenu(hwnd), CM_RESERVAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODARESERVADA, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_BLOQUEAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESBLOQUEAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_LIBERAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESASIGNA, MF_ENABLED | MF_BYCOMMAND);
         for(i=CM_SOLOLECTURA; i <= CM_SINACCESO; i++)
            EnableMenuItem(GetMenu(hwnd), i, MF_ENABLED | MF_BYCOMMAND);
         break;
      case bloqueado:
         EnableMenuItem(GetMenu(hwnd), CM_RESERVAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_ACOMODARESERVADA, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_BLOQUEAR, MF_GRAYED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESBLOQUEAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_LIBERAR, MF_ENABLED | MF_BYCOMMAND);
         EnableMenuItem(GetMenu(hwnd), CM_DESASIGNA, MF_ENABLED | MF_BYCOMMAND);
         for(i=CM_SOLOLECTURA; i <= CM_SINACCESO; i++)
            EnableMenuItem(GetMenu(hwnd), i, MF_ENABLED | MF_BYCOMMAND);
         break;
   }
}

void ActualizarVentana(HDC hdc, DATOS *Datos)
{
   MEMORY_BASIC_INFORMATION mbi;
   char cad[256];

   SetBkMode(hdc, TRANSPARENT);
   VirtualQuery(Datos, &mbi, sizeof(MEMORY_BASIC_INFORMATION));
   sprintf(cad, "Base Address: %p", mbi.BaseAddress);
   TextOut(hdc, 10, 10, cad, strlen(cad));
   sprintf(cad, "Allocation Base: %p", mbi.AllocationBase);
   TextOut(hdc, 10, 30, cad, strlen(cad));
   sprintf(cad, "Allocation Protect (%lx):", mbi.AllocationProtect);
   if(mbi.AllocationProtect & PAGE_READONLY) strcat(cad, " READONLY");
   else if(mbi.AllocationProtect & PAGE_READWRITE) strcat(cad, " READWRITE");
   else if(mbi.AllocationProtect & PAGE_WRITECOPY) strcat(cad, " WRITECOPY");
   else if(mbi.AllocationProtect & PAGE_EXECUTE) strcat(cad, " EXECUTE");
   else if(mbi.AllocationProtect & PAGE_EXECUTE_READ) strcat(cad, " EXECUTE_READ");
   else if(mbi.AllocationProtect & PAGE_EXECUTE_READWRITE) strcat(cad, " EXECUTE_READWRITE");
   else if(mbi.AllocationProtect & PAGE_EXECUTE_WRITECOPY) strcat(cad, " EXECUTE_WRITECOPY");
   else if(mbi.AllocationProtect & PAGE_NOACCESS) strcat(cad, " NOACCESS");
   else strcat(cad, " DESCONOCIDO");
   if(mbi.AllocationProtect & PAGE_GUARD) strcat(cad, " GUARD");
   else if(mbi.AllocationProtect & PAGE_NOCACHE) strcat(cad, " NOCACHE");
   TextOut(hdc, 10, 50, cad, strlen(cad));
   sprintf(cad, "Region Size: %lx", mbi.RegionSize);
   TextOut(hdc, 10, 70, cad, strlen(cad));
   strcpy(cad, "Estado:");
   if(mbi.State & MEM_COMMIT) strcat(cad, " MEM_COMMIT");
   if(mbi.State & MEM_FREE) strcat(cad, " MEM_FREE");
   if(mbi.State & MEM_RESERVE) strcat(cad, " MEM_RESERVE");
   //else strcat(cad, "DESCONOCIDO");
   TextOut(hdc, 10, 90, cad, strlen(cad));
   sprintf(cad, "Protect (%lx):", mbi.Protect);
   if(mbi.Protect & PAGE_READONLY) strcat(cad, " READONLY");
   else if(mbi.Protect & PAGE_READWRITE) strcat(cad, " READWRITE");
   else if(mbi.Protect & PAGE_WRITECOPY) strcat(cad, " WRITECOPY");
   else if(mbi.Protect & PAGE_EXECUTE) strcat(cad, " EXECUTE");
   else if(mbi.Protect & PAGE_EXECUTE_READ) strcat(cad, " EXECUTE_READ");
   else if(mbi.Protect & PAGE_EXECUTE_READWRITE) strcat(cad, " EXECUTE_READWRITE");
   else if(mbi.Protect & PAGE_EXECUTE_WRITECOPY) strcat(cad, " EXECUTE_WRITECOPY");
   else if(mbi.Protect & PAGE_NOACCESS) strcat(cad, " NOACCESS");
   else strcat(cad, " DESCONOCIDO");
   if(mbi.Protect & PAGE_GUARD) strcat(cad, " GUARD");
   else if(mbi.Protect & PAGE_NOCACHE) strcat(cad, " NOCACHE");
   TextOut(hdc, 10, 110, cad, strlen(cad));
   sprintf(cad, "Tipo(%lx):", mbi.Type);
   if(mbi.Type & MEM_IMAGE) strcat(cad, " IMAGE");
   else if(mbi.Type & MEM_MAPPED) strcat(cad, " MAPPED");
   else if(mbi.Type & MEM_PRIVATE) strcat(cad, " PRIVATE");
   else strcat(cad, " DESCONOCIDO");
   TextOut(hdc, 10, 130, cad, strlen(cad));
}
